#!/bin/sh

# Copyright  2013, Trond Endrestl <Trond.Endrestol@ximalas.info>
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

if [ -z "${ROOTPOOL}" ]; then
  echo "$0: you must set the environment variable ROOTPOOL to the name of your root pool" >/dev/stderr;
  echo "$0: e.g. export ROOTPOOL=zroot" >/dev/stderr;
  exit 69;
fi

if ! zpool list "${ROOTPOOL}" >/dev/null 2>/dev/null; then
  echo "$0: root pool ${ROOTPOOL} does not exist" >/dev/stderr;
  echo "$0: try: zpool create -o cachefile=/tmp/zpool.cache -O mountpoint=legacy ${ROOTPOOL} ..." >/dev/stderr;
  exit 69;
fi

if [ -z "${DATAPOOL}" ]; then
  echo "$0: you must set the environment variable DATAPOOL to the name of your data pool" >/dev/stderr;
  echo "$0: e.g. export DATAPOOL=zdata" >/dev/stderr;
  echo "$0: your data pool can be the same as your root pool" >/dev/stderr;
  echo "$0: e.g. export DATAPOOL=\${ROOTPOOL}" >/dev/stderr;
  exit 69;
fi

if [ "${DATAPOOL}" != "${ROOTPOOL}" ]; then
  if ! zpool list "${DATAPOOL}" >/dev/null 2>/dev/null; then
    echo "$0: data pool ${DATAPOOL} does not exist" >/dev/stderr;
    echo "$0: try: zpool create -o cachefile=/tmp/zpool.cache -O mountpoint=legacy ${DATAPOOL} ..." >/dev/stderr;
    exit 69;
  fi;
fi

if [ -z "${RELEASEDATE}" ]; then
  echo "$0: you must set the environment variable RELEASEDATE to the date of the release you are installing" >/dev/stderr;
  echo "$0: e.g. export RELEASEDATE=20130602" >/dev/stderr;
  exit 69;
fi

if [ -z "${RELEASEREV}" ]; then
  echo "$0: you must set the environment variable RELEASEREV to the Subversion revision number of the release you are installing" >/dev/stderr;
  echo "$0: e.g. export RELEASEREV=251259" >/dev/stderr;
  exit 69;
fi

# It is assumed the pools were created with -O mountpoint=legacy.

zfs create ${ROOTPOOL}/ROOT
zfs create ${ROOTPOOL}/ROOT/${RELEASEDATE}-r${RELEASEREV}

zpool set bootfs=${ROOTPOOL}/ROOT/${RELEASEDATE}-r${RELEASEREV} ${ROOTPOOL}

zfs create ${ROOTPOOL}/tmp
zfs create ${ROOTPOOL}/usr
zfs create ${ROOTPOOL}/usr/compat
zfs create ${ROOTPOOL}/usr/compat/linux
zfs create ${ROOTPOOL}/usr/local
zfs create ${ROOTPOOL}/usr/local/certs
zfs create ${ROOTPOOL}/usr/local/etc
zfs create ${ROOTPOOL}/usr/local/etc/namedb
zfs create ${ROOTPOOL}/usr/obj
zfs create ${ROOTPOOL}/usr/ports
zfs create ${ROOTPOOL}/usr/ports/distfiles
zfs create ${ROOTPOOL}/usr/ports/packages
zfs create ${ROOTPOOL}/usr/ports/workdirs
zfs create ${ROOTPOOL}/usr/src
zfs create ${ROOTPOOL}/var
zfs create ${ROOTPOOL}/var/backups
zfs create ${ROOTPOOL}/var/crash
zfs create ${ROOTPOOL}/var/db
zfs create ${ROOTPOOL}/var/db/pkg
zfs create ${ROOTPOOL}/var/db/ports
zfs create ${ROOTPOOL}/var/empty
zfs create ${ROOTPOOL}/var/log
zfs create ${ROOTPOOL}/var/mail
zfs create ${ROOTPOOL}/var/named
zfs create ${ROOTPOOL}/var/run
zfs create ${ROOTPOOL}/var/spool
zfs create ${ROOTPOOL}/var/tmp
zfs create ${ROOTPOOL}/var/unbound

zfs create ${DATAPOOL}/home

if [ "${DATAPOOL}" != "${ROOTPOOL}" ]; then
  zfs create ${DATAPOOL}/usr;
  zfs create ${DATAPOOL}/usr/local;
fi
zfs create ${DATAPOOL}/usr/local/pgsql
zfs create ${DATAPOOL}/usr/local/www

if [ "${DATAPOOL}" != "${ROOTPOOL}" ]; then
  zfs create ${DATAPOOL}/var;
  zfs create ${DATAPOOL}/var/db;
  zfs create ${DATAPOOL}/var/spool;
fi
zfs create ${DATAPOOL}/var/db/mysql
zfs create ${DATAPOOL}/var/spool/ftp

zfs set atime=off ${DATAPOOL}/var/spool/ftp;

zfs set compression=gzip-9 ${ROOTPOOL}/tmp
zfs set compression=gzip-9 ${ROOTPOOL}/usr/local/certs
zfs set compression=gzip-9 ${ROOTPOOL}/usr/local/etc
zfs set compression=gzip-9 ${DATAPOOL}/usr/local/www
zfs set compression=gzip-9 ${ROOTPOOL}/usr/ports
zfs set compression=gzip-9 ${ROOTPOOL}/usr/src
zfs set compression=gzip-9 ${ROOTPOOL}/var/db/pkg
zfs set compression=gzip-9 ${ROOTPOOL}/var/db/ports
zfs set compression=gzip-9 ${ROOTPOOL}/var/log
zfs set compression=gzip-9 ${ROOTPOOL}/var/mail
zfs set compression=gzip-9 ${ROOTPOOL}/var/named
zfs set compression=gzip-9 ${ROOTPOOL}/var/tmp
zfs set compression=gzip-9 ${ROOTPOOL}/var/unbound

zfs set compression=off ${ROOTPOOL}/usr/ports/distfiles
zfs set compression=off ${ROOTPOOL}/usr/ports/packages
zfs set compression=off ${ROOTPOOL}/usr/ports/workdirs

zfs set exec=off ${ROOTPOOL}/usr/local/etc/namedb
zfs set exec=off ${ROOTPOOL}/usr/ports/distfiles
zfs set exec=off ${ROOTPOOL}/usr/ports/packages
zfs set exec=off ${ROOTPOOL}/usr/src
zfs set exec=off ${ROOTPOOL}/var/backups
zfs set exec=off ${ROOTPOOL}/var/crash
zfs set exec=off ${ROOTPOOL}/var/db
zfs set exec=off ${ROOTPOOL}/var/empty
zfs set exec=off ${ROOTPOOL}/var/log
zfs set exec=off ${ROOTPOOL}/var/mail
zfs set exec=off ${ROOTPOOL}/var/named
zfs set exec=off ${ROOTPOOL}/var/run
zfs set exec=off ${ROOTPOOL}/var/spool
zfs set exec=off ${ROOTPOOL}/var/unbound

if [ "${DATAPOOL}" != "${ROOTPOOL}" ]; then
  zfs set exec=off ${DATAPOOL}/var/spool;
fi

zfs set exec=on ${ROOTPOOL}/var/db/pkg
zfs set exec=on ${ROOTPOOL}/var/db/ports

zfs set setuid=off ${ROOTPOOL}/tmp
zfs set setuid=off ${DATAPOOL}/home
zfs set setuid=off ${ROOTPOOL}/usr/local/etc/namedb
zfs set setuid=off ${ROOTPOOL}/usr/ports
zfs set setuid=off ${ROOTPOOL}/usr/src
zfs set setuid=off ${ROOTPOOL}/var/backups
zfs set setuid=off ${ROOTPOOL}/var/crash
zfs set setuid=off ${ROOTPOOL}/var/db
zfs set setuid=off ${ROOTPOOL}/var/empty
zfs set setuid=off ${ROOTPOOL}/var/log
zfs set setuid=off ${ROOTPOOL}/var/mail
zfs set setuid=off ${ROOTPOOL}/var/named
zfs set setuid=off ${ROOTPOOL}/var/run
zfs set setuid=off ${ROOTPOOL}/var/spool
zfs set setuid=off ${ROOTPOOL}/var/tmp
zfs set setuid=off ${ROOTPOOL}/var/unbound

if [ "${DATAPOOL}" != "${ROOTPOOL}" ]; then
  zfs set setuid=off ${DATAPOOL}/var/spool;
fi

zfs set recordsize=8K  ${DATAPOOL}/usr/local/pgsql
zfs set recordsize=16K ${DATAPOOL}/var/db/mysql
